# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...errors.unauthorized_error import UnauthorizedError
from ...types.rest_err_response import RestErrResponse
from ...types.v_2_api_key_response_envelope import V2ApiKeyResponseEnvelope
from ...types.v_2_list_users_envelope import V2ListUsersEnvelope
from ...types.v_2_user_envelope import V2UserEnvelope

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(self, org_id: str) -> V2ListUsersEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - org_id: str.
        ---
        from polytomic.client import Polytomic

        client = Polytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        client.users.list(
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2ListUsersEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, org_id: str, *, email: str, role: typing.Optional[str] = OMIT) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - org_id: str.

            - email: str.

            - role: typing.Optional[str].
        ---
        from polytomic.client import Polytomic

        client = Polytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        client.users.create(
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
            email="email",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"email": email}
        if role is not OMIT:
            _request["role"] = role
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, org_id: str) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - id: str.

            - org_id: str.
        ---
        from polytomic.client import Polytomic

        client = Polytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        client.users.get(
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def remove(self, id: str, org_id: str) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - id: str.

            - org_id: str.
        ---
        from polytomic.client import Polytomic

        client = Polytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        client.users.remove(
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(self, id: str, org_id: str, *, email: str, role: typing.Optional[str] = OMIT) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - id: str.

            - org_id: str.

            - email: str.

            - role: typing.Optional[str].
        ---
        from polytomic.client import Polytomic

        client = Polytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        client.users.update(
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
            email="email",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"email": email}
        if role is not OMIT:
            _request["role"] = role
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_api_key(self, org_id: str, id: str, *, force: typing.Optional[bool] = None) -> V2ApiKeyResponseEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - org_id: str.

            - id: str.

            - force: typing.Optional[bool].
        ---
        from polytomic.client import Polytomic

        client = Polytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        client.users.create_api_key(
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}/keys"
            ),
            params=remove_none_from_dict({"force": force}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2ApiKeyResponseEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(self, org_id: str) -> V2ListUsersEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - org_id: str.
        ---
        from polytomic.client import AsyncPolytomic

        client = AsyncPolytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        await client.users.list(
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2ListUsersEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, org_id: str, *, email: str, role: typing.Optional[str] = OMIT) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - org_id: str.

            - email: str.

            - role: typing.Optional[str].
        ---
        from polytomic.client import AsyncPolytomic

        client = AsyncPolytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        await client.users.create(
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
            email="email",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"email": email}
        if role is not OMIT:
            _request["role"] = role
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, org_id: str) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - id: str.

            - org_id: str.
        ---
        from polytomic.client import AsyncPolytomic

        client = AsyncPolytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        await client.users.get(
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def remove(self, id: str, org_id: str) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - id: str.

            - org_id: str.
        ---
        from polytomic.client import AsyncPolytomic

        client = AsyncPolytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        await client.users.remove(
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(self, id: str, org_id: str, *, email: str, role: typing.Optional[str] = OMIT) -> V2UserEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - id: str.

            - org_id: str.

            - email: str.

            - role: typing.Optional[str].
        ---
        from polytomic.client import AsyncPolytomic

        client = AsyncPolytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        await client.users.update(
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
            email="email",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"email": email}
        if role is not OMIT:
            _request["role"] = role
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2UserEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_api_key(
        self, org_id: str, id: str, *, force: typing.Optional[bool] = None
    ) -> V2ApiKeyResponseEnvelope:
        """
        > 🚧 Requires partner key
        >
        > User endpoints are only accessible using [partner keys](https://docs.polytomic.com/reference/authentication#partner-keys)

        Parameters:
            - org_id: str.

            - id: str.

            - force: typing.Optional[bool].
        ---
        from polytomic.client import AsyncPolytomic

        client = AsyncPolytomic(
            polytomic_version="YOUR_POLYTOMIC_VERSION",
            token="YOUR_TOKEN",
        )
        await client.users.create_api_key(
            org_id="248df4b7-aa70-47b8-a036-33ac447e668d",
            id="248df4b7-aa70-47b8-a036-33ac447e668d",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/organizations/{org_id}/users/{id}/keys"
            ),
            params=remove_none_from_dict({"force": force}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(V2ApiKeyResponseEnvelope, _response.json())  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(RestErrResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
